/* File BRMDYN.C   Broom dynamics  */

#ifndef BRMDYN_C
#define BRMDYN_C

#include <math.h>
#include "brm.h"

void new_broom_state (float frce,state_rec old_state,state_rec *new_state);
void force_effect (float force,state_rec in_state,state_rec *out_state);
void set_broom_at_start_position (state_rec *state);

/* Implementation  */

const float
    l = 0.5, /* Distance from pivot to broom center of mass (m) */
    m = 1.1, /* Mass of cart plus broom (kg) */
    mb = 0.1, /* Mass of broom (kg) */
    g = 9.8, /* Gravitational constant (m/(sec*sec)) */
    four_thirds = 1.3333333;

float f_theta (float frce,float th1,float th2)
   {
   float denom,numer,cost,sint;
   cost = cos (th1);
   sint = sin (th1);
   denom = four_thirds * m * l - mb * l * cost * cost;  /* Always > 0 */
   numer = m * g * sint -
	    cost * (frce + mb * l * th2 * th2 * sint);
   return numer/denom;
   }

float  f_x (float frce,float th1,float th2,float th3)
   {
   float cost,sint,term;
   cost = cos (th1);
   sint = sin (th1);
   term = mb * l * (th2 * th2 * sint - th3 * cost);
   return (frce + term)/m;
   }

void new_broom_state (float frce,state_rec old_state,state_rec *new_state)
   {
    const float h = 0.02;
    float th3;
    /* Euler's method applied to system of equations */
    /* (not known for accuracy, but good enough here !) */
    new_state->theta = old_state.theta + h * old_state.theta_dot;
    th3 = f_theta (frce,old_state.theta,old_state.theta_dot);
    new_state->theta_dot = old_state.theta_dot + h * th3;
    new_state->x_pos = old_state.x_pos + h * old_state.x_dot;
    new_state->x_dot = old_state.x_dot +
	h * f_x (frce,old_state.theta,old_state.theta_dot,th3);
    return;
    }

void force_effect (float force,state_rec in_state,state_rec *out_state)
    {
    state_rec intermed_state;
    /* Need more than one iteration of broom state to see effect of force */
    new_broom_state (force,in_state,&intermed_state);
    new_broom_state (0.0,intermed_state,out_state);
    return;
    }

void set_broom_at_start_position (state_rec *state)
    {
    state->x_pos = 0.0;
    state->x_dot = 0.0;
    state->theta = 0.0;
    state->theta_dot = 0.0;
    return;
    }

#endif

