
/* File BRMGRCN.C  Screen constants for broom graphics  */
/*                 Constants in terms of screen inches  */

#ifndef BRMGRCN_C
#define BRMGRCN_C

#include <graphics.h>

void set_pixel_positions (void);
void set_video_mode_and_colors (int video_driver,int *video_mode);

/*  Implementation  */

const float
   /* Units are screen inches, assuming a 10 in wide, 7 in high screen */
   screen_width_in = 10.0,
   screen_height_in = 7.0,
   window_left_in = 3.6,
   window_right_in = 8.6,
   window_top_in = 1.8,
   window_bottom_in = 5.3,
   cart_half_width_in = 0.5,
   half_wheelbase_in = 0.265625,
   cart_ht_in = 0.40,
   axle_ht_in = 0.16,
   wheel_radius_in = 0.125,
   pivot_ht_in = 0.1,
   zero_pos_in = 3.86,
   thrm_width_in = 0.3125,
   f_thrm_ht_in = 0.3125,
   broom_half_width_in = 0.078125,
   broom_length_in = 2.0;

int
   screen_max_cols,
   screen_max_rows,
   window_left,
   window_right,
   window_top,
   window_bottom,
   window_ht,
   window_width,
   cart_half_width,
   half_wheelbase,
   cart_ht,
   axle_ht,
   cart_top,
   cart_bottom,
   wheel_radius,
   pivot_ht,
   pivot_top,
   pivot_bottom,
   broom_half_width,
   broom_length_h,
   broom_length_v,
   zero_pos,
   f_thrm_width,
   f_thrm_ht,
   f_thrm_half_width,
   f_thrm_left,
   f_thrm_right,
   f_thrm_top,
   f_thrm_bottom,
   f_zero_pos,
   cart_color,
   border_color,
   wheel_color,
   wheel_rim_color,
   broom_color,
   pivot_color,
   pivot_pt_color,
   text_color,
   f_thrm_back_color,
   f_thrm_fore_color;


void set_pixel_positions (void)
   {
   screen_max_cols = getmaxx() + 1;
   screen_max_rows = getmaxy() + 1;
   window_left = (window_left_in/screen_width_in) * screen_max_cols - 1;
   window_right = (window_right_in/screen_width_in) * screen_max_cols - 1;
   window_top = (window_top_in/screen_height_in) * screen_max_rows - 1;
   window_bottom =   (window_bottom_in/screen_height_in) *
		  screen_max_rows - 1;
   window_width = window_right - window_left;
   window_ht = window_bottom - window_top;
   cart_half_width =   (cart_half_width_in/screen_width_in) *
		  screen_max_cols;
   broom_half_width =   (broom_half_width_in/screen_width_in) *
		  screen_max_cols;
   half_wheelbase =   (half_wheelbase_in/screen_width_in) *
		  screen_max_cols;
   cart_ht =   (cart_ht_in/screen_height_in) * screen_max_rows;
   axle_ht =   (axle_ht_in/screen_height_in) * screen_max_rows;
   wheel_radius =   (wheel_radius_in/screen_width_in) * screen_max_cols;
   broom_length_h =   (broom_length_in/screen_width_in) *
		     screen_max_cols;
   broom_length_v =   (broom_length_in/screen_height_in) *
		     screen_max_rows;
   pivot_ht =   (pivot_ht_in/screen_height_in) * screen_max_rows;
   zero_pos =   (zero_pos_in/screen_height_in) * screen_max_rows;
   cart_top = window_ht - axle_ht - cart_ht;
   cart_bottom = window_ht - axle_ht;
   pivot_bottom = cart_top - 1;
   pivot_top = pivot_bottom - pivot_ht;
   f_thrm_ht =   (f_thrm_ht_in/screen_height_in) * screen_max_rows;
   f_thrm_width = (window_right - window_left) /2;
   f_thrm_half_width = f_thrm_width /2;
   f_thrm_left = window_left + f_thrm_half_width;
   f_thrm_right = f_thrm_left + f_thrm_width;
   f_thrm_top = window_bottom + 20;
   f_thrm_bottom = f_thrm_top + f_thrm_ht;
   f_zero_pos = f_thrm_left + f_thrm_half_width;
   }

void set_cga_color (unsigned short color)
   /* Only 1 color available in CGA Hi.  Use this to turn color on or off. */
   {
   cart_color = color;
   wheel_color = color;
   broom_color = color;
   pivot_color = color;
   pivot_pt_color = broom_color;
   f_thrm_fore_color = color;
   text_color = color;
   }

void set_video_mode_and_colors (int video_driver,int *video_mode)
   {
   switch (video_driver) {
case VGA:
   *video_mode = VGAMED;  /* Use medium so we have two pages */
   break;
case EGA:
   *video_mode = EGAHI;
   break;
case CGA:
   *video_mode = CGAHI;
   break;
   }  /* end switch */
   setgraphmode (*video_mode);

   if ((video_driver == VGA) || (video_driver == EGA))
      {
      border_color = LIGHTBLUE;
      cart_color = LIGHTGREEN;
      wheel_color = LIGHTGRAY;
      broom_color = LIGHTMAGENTA;
      pivot_color = YELLOW;
      pivot_pt_color = broom_color;
      f_thrm_back_color = RED;
      f_thrm_fore_color = LIGHTRED;
      text_color = YELLOW;
      }
   else   /* CGA */
      {
      set_cga_color (1);
      border_color = 1;
      f_thrm_back_color = 1;
      }
   }  /* end function */

#endif