/*  File DYN.H Dynamical Systems Header File  */

#ifndef DYN_H
#define DYN_H

#include "ff.h"
#include "utgr.h"

#define DESCR_LEN 8

typedef enum {Henon,Ikeda} dyn_sys_type;
typedef char descr_str[DESCR_LEN];

extern const  descr_str dyn_descr[2]; /* Set in DYNAMSYS.C  */

typedef struct {
      float a,b;
      } henon_param;

typedef struct {
      float alpha,gamma,kappa,r;
      } ikeda_param;

typedef struct {
      dyn_sys_type the_sys;
      descr_str sys_descr;
      henon_param hn;
      ikeda_param ik;
      int no_of_items;
      graph_setup_rec gr_rec;
      } dyn_setup_rec;

typedef  void (*dyn_sys_proc) (float x_in,float y_in,float *x_out,
    float *y_out);

/*  DYNGRAPH.C  */
void graph_dynamical_system (graph_setup_rec *the_graph_setup,
     dyn_sys_proc dyn_sys);

/*  DYNAMSYS.C  */
extern dyn_setup_rec g_dyn_setup;
void dyn_f (float x_in,float y_in,float *x_out,float *y_out);
void display_dyn_param (dyn_setup_rec the_rec,short *line);
void initialize_dyn_setup (dyn_setup_rec *the_rec);

/*  DYNAMRUN.C  */
void run_dynam_sys(dyn_setup_rec setup);

/*  DYNFFRUN.C  */
void run_dynamic_network (ff_setup_rec setup,dyn_setup_rec dyn_setup);

/*  DYNCREAT.C  */
void create_training_file (ff_setup_rec setup,dyn_setup_rec dyn_rec);

#endif