/*  FFDISPL.C Procedures to display neural net  */
/*            training progress on screeen. */

#ifndef FFDISPL_C
#define FFDISPL_C

#include <conio.h>
#include <math.h>

#include "uttypes.h"
#include "ff.h"

short start_hor_pos,display_factor;

int scale (float value);
void display_results (ff_setup_rec setup,display_rec display,
     f_vector d_out,f_vector net_out);
void setup_screen (ff_setup_rec setup,display_rec display);
void reset_screen (display_rec display);

/* Implementation  */

int scale (float value) {
   int scale_val;
   scale_val = ceil (value * 5 - 0.5);
   if (scale_val < 1) return 0;
   if (scale_val > 5) return 5;
   return scale_val;
   }  /* end function */

void setup_screen (ff_setup_rec setup,display_rec display) {
     clrscr ();
     textcolor (WHITE);
     gotoxy (1,1);
     cprintf ("Iteration: %ld",display.iter);
     if (setup.shuffle) {
	gotoxy (28,1);
	cputs ("Shuffled Input");
	}
     gotoxy (49,1);
     cprintf ("Error Threshold: %8.6f",setup.error_threshold);
     gotoxy (10,22);
     cprintf ("High water mark: %3d Right (out of %d).",
	display.high_water,display.nbr_tr_items);
     textcolor (YELLOW);
     if (setup.no_of_outputs <= 40) display_factor = 2;
     else display_factor = 1;
     start_hor_pos = 35 - (setup.no_of_outputs/2) * display_factor;
     if (start_hor_pos < 1) start_hor_pos = 1;
     gotoxy (start_hor_pos,7);
     cputs ("Desired output: ");
     gotoxy (start_hor_pos,14);
     cputs ("Error: ");
     return;
     }  /* end procedure */

void reset_screen(display_rec display) {
     gotoxy (12,1);
     cprintf ("%ld",display.iter);
     gotoxy (27,22);
     cprintf ("%3d",display.high_water);
     gotoxy (10,24);
     cputs ("                      ");
     gotoxy (1,25);
     return;
     }  /* end procedure */

void display_results (ff_setup_rec setup,display_rec display,
     f_vector d_out,f_vector net_out) {
     int i,j,k,hor_pos;
     gotoxy (25,3);
     cprintf ("Right:  %3d              Wrong: %3d",display.right_count,
	  display.wrong_count);
     gotoxy (20,5);
     cprintf ("Last Right:  %3d         Last Wrong: %3d",
	  display.last_right,display.last_wrong);
     for (i = 1;i<=setup.no_of_outputs;i++) {
	 k = scale (d_out[i]);
	 hor_pos = start_hor_pos + display_factor*i;
	 for (j = 1;j <= 5;j++) {
	     gotoxy (hor_pos,13 - j);
	     putch (BLOCK_1);
	     } /* j */
	 for (j = 1;j <= k;j++) {
	     gotoxy (hor_pos,13 - j);
	     putch (BLOCK_4);
	     } /* j */

	 k = scale (fabs (d_out[i] - net_out[i]));
	 for (j = 1;j <= 5;j++) {
	     gotoxy (hor_pos,20 - j);
	     putch (BLOCK_1);
	     } /* j */
	 for (j = 1;j<=k;j++) {
	     gotoxy (hor_pos,20 - j);
	     putch (BLOCK_4);
	     } /* j */
	 } /* i */
	 gotoxy (1,25);
	 }  /* end proc */

#endif