// File FFSETUP.CPP Set up Turbo Vision interface for
//      Feedforward Neural Networks
//      Setups for specific feedforward network applications will be
//      derived from this.

#ifndef FFSETUP_CPP
#define FFSETUP_CPP

#include <conio.h>
#include <string.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <io.h>
#include <math.h>

#include "uisetup.cpp"
#include "uttypes.h"

extern "C" {
  #include "ff.h"
  }

weights_rec g_system_weights;
ff_setup_rec g_ff_setup;

class tffsetapp : public TSetApp
   {
   public :
     tffsetapp();
     virtual void setup();
     static TMenuBar *initMenuBar (TRect r );
     virtual char *save_setup (char *the_file_name);
     virtual char *read_setup (char *the_file_name);
   };

#define MAX_NO_OF_NODES 1000

// We'll need to refer to these later as item numbers
int set_shuffle,set_in_weights_from_file,
    set_test_file_name,set_data_file_name,set_new_data_file_name,
    set_norm_in_min,set_norm_out_min;

void ff_setup_to_collection (ff_setup_rec *the_ff_setup,
   TDataCollection *the_collection)
   {
      void * value_ptr = NULL;
      tdata_type the_type;
      unsigned short len;
      boolean the_check = TRUE;
      unsigned short dec_places = 0;
      float the_max = 0.0;
      float the_min = 0.0;
      boolean enable = TRUE;
      int item_no = 0;

   value_ptr = &the_ff_setup->error_threshold;
   the_type = REAL_DATA;
   len = 8;
   dec_places = 6;
   the_max = 1.0;
   the_min = 1e-6;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Error Threshold",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_ff_setup->start_iter;
   the_type = INT_DATA;
   len = 5;
   dec_places = 0;
   the_max = 65535;
   the_min = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Starting Iteration",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_ff_setup->shuffle;
   the_type = BOOLEAN_DATA;
   len = 3;
   dec_places = 0;
   the_max = 0;
   the_min = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Shuffle Input Order?",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
   set_shuffle = item_no;

   value_ptr = &the_ff_setup->no_of_inputs;
   the_type = INT_DATA;
   len = 5;
   dec_places = 0;
   the_max = MAX_NO_OF_NODES;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Inputs",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_ff_setup->no_of_h1_nodes;
   the_type = INT_DATA;
   len = 5;
   dec_places = 0;
   the_max = MAX_NO_OF_NODES;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Hidden Nodes",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_ff_setup->no_of_outputs;
   the_type = INT_DATA;
   len = 5;
   dec_places = 0;
   the_max = MAX_NO_OF_NODES;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Outputs",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_ff_setup->in_weights_from_file;
   the_type = BOOLEAN_DATA;
   len = 3;
   dec_places = 0;
   the_max = 0;
   the_min = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Input Wts from File?",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
   set_in_weights_from_file = item_no;

   value_ptr = &the_ff_setup->files.in_wts_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Input Wts File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.WTS",item_no));

   value_ptr = &the_ff_setup->files.out_wts_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Output Wts File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.WTS",item_no));

   value_ptr = &the_ff_setup->files.training_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Training File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.TRN",item_no));

   value_ptr = &the_ff_setup->files.test_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Test File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.TST",item_no));
   set_test_file_name = item_no;

   value_ptr = &the_ff_setup->files.data_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Data File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.DAT",item_no));
   set_data_file_name = item_no;

   value_ptr = &the_ff_setup->files.new_data_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Edited Data File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.DAT",item_no));
   set_new_data_file_name = item_no;

   value_ptr = &the_ff_setup->sigmoid_out;
   the_type = BOOLEAN_DATA;
   len = 3;
   dec_places = 0;
   the_max = 0;
   the_min = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Sigmoid Output Layer",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_ff_setup->alpha;
   the_type = REAL_DATA;
   len = 7;
   dec_places = 4;
   the_max = 10.0;
   the_min = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Alpha",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_ff_setup->learning_rate;
   the_type = REAL_DATA;
   len = 7;
   dec_places = 4;
   the_max = 10.0;
   the_min = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Learning Rate",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_ff_setup->norm_in_min;
   the_type = REAL_DATA;
   len = 7;
   dec_places = 4;
   the_max = 0.0;
   the_min = -10.0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Norm Inp Min",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
   set_norm_in_min = item_no;

   value_ptr = &the_ff_setup->norm_out_min;
   the_type = REAL_DATA;
   len = 7;
   dec_places = 4;
   the_max = 0.5;
   the_min = 0.0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Norm Out Min",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
   set_norm_out_min = item_no;

   value_ptr = &the_ff_setup->weight_range;
   the_type = REAL_DATA;
   len = 7;
   dec_places = 4;
   the_max = 10.0;
   the_min = 0.0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Weight Range",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
   return;
   }

tffsetapp::tffsetapp() : TSetApp(),
   TProgInit (&tffsetapp::initStatusLine,
	      &tffsetapp::initMenuBar,
	      &tffsetapp::initDeskTop
	      )
   {
   ff_setup_to_collection (&g_ff_setup,system_setup_data.item_collection);
   if (quit_flag == startup) about_dialog();
   }

TMenuBar *tffsetapp::initMenuBar ( TRect r )
{
   r.b.y = r.a.y + 1;  //Set bottom line 1 line below top line
   init_submenus();
   return new TMenuBar (r,new TMenu(*about_submenu));
 }

void ff_setup_processing(tlist_box_data *the_data_rec,int item_no)
   {
   ((tcolor_data_obj *) (the_data_rec->item_collection)->at(item_no - 1))
     ->get_new_value();
   if (item_no == set_norm_in_min)
      g_ff_setup.norm_in_range = 2 * fabs(g_ff_setup.norm_in_min);
   if (item_no == set_norm_out_min)
      g_ff_setup.norm_out_range = 1.0 - 2 * g_ff_setup.norm_out_min;
   return;
   }

void tffsetapp::setup()
   {
   list_dialog ("SETUP PARAMETERS","",&system_setup_data,
     MAX_NO_OF_SETUP_ITEMS,&ff_setup_processing);
   return;
   }

char *tffsetapp::save_setup (char *the_file_name)
   {
   write_record (the_file_name,(void *)&g_ff_setup,sizeof(ff_setup_rec));
   return the_file_name;
   }

char *tffsetapp::read_setup (char *the_file_name)
   {
   read_record (the_file_name,(void *)&g_ff_setup,sizeof(ff_setup_rec));
   return the_file_name;
   }

void initialize_ff_setup (ff_setup_rec *the_setup) {
   ui_files = (file_names_rec *)&(the_setup->files);
   strcpy(the_setup->files.setup_file_name,"");
   strcpy(the_setup->files.training_setup_file_name,"");
   strcpy(the_setup->files.in_wts_file_name,"");
   strcpy(the_setup->files.out_wts_file_name,"");
   strcpy(the_setup->files.training_file_name,"");
   strcpy(the_setup->files.test_file_name,"");
   strcpy(the_setup->files.data_file_name,"");
   strcpy(the_setup->files.new_data_file_name,"");
   the_setup->error_threshold = 0.01;
   the_setup->start_iter = 0;
   the_setup->shuffle = TRUE;
   the_setup->no_of_inputs = 0;
   the_setup->no_of_h1_nodes = 0;
   the_setup->no_of_outputs = 0;
   the_setup->in_weights_from_file = FALSE;
   the_setup->sigmoid_out = TRUE;
   the_setup->alpha = 0.9;
   the_setup->learning_rate = 1.0;
   the_setup->norm_in_min = -5.0;
   the_setup->norm_in_range = 10.0;
   the_setup->norm_out_min = 0.05;
   the_setup->norm_out_range = 0.95;
   the_setup->weight_range = 1.0;
   return;
   }

#endif