/* File HOPWTS.C  Read, write, compute Hopfield weights  */

#ifndef HOPWTS_C
#define HOPWTS_C

#include <stdlib.h>
#include <conio.h>
#include <stdio.h>
#include <string.h>

#include "uttypes.h"
#include "hop.h"

void initialize_hop_setup (hop_setup_rec *the_setup,
   pattern_display_data *disp_data);
short read_xmp_patterns (hop_setup_rec *the_setup,
   pattern_display_data *disp_data,path_str the_file_name,
   pattern_array the_patterns);
int **allocate_2d_int_matrix (int rows_lower_lim,int rows_upper_lim,
   int cols_lower_lim,int cols_upper_lim);
void free_2d_int_matrix (int **a,int rows_lower_lim,int cols_lower_lim);
void compute_hop_weights (pattern_display_data *disp_data,
   pattern_array the_patterns,int **weights);
short write_hop_weights (path_str file_name, int **w,int no_of_nodes);
short read_hop_weights (path_str file_name,int **w,int no_of_nodes);

/* Implementation */

void initialize_hop_setup (hop_setup_rec *the_setup,
   pattern_display_data *disp_data) {
   ui_files = (file_names_rec *)&(the_setup->files);
   strcpy(the_setup->files.setup_file_name,"");
   strcpy(the_setup->files.training_setup_file_name,"");
   strcpy(the_setup->files.in_wts_file_name,"");
   strcpy(the_setup->files.out_wts_file_name,"");
   strcpy(the_setup->files.training_file_name,"");
   strcpy(the_setup->files.test_file_name,"");
   strcpy(the_setup->files.data_file_name,"");
   strcpy(the_setup->files.new_data_file_name,"");
   the_setup->no_of_patterns = 5;
   the_setup->no_of_rows = 10;
   the_setup->no_of_cols = 10;
   disp_data->no_of_patterns = the_setup->no_of_patterns;
   disp_data->no_of_rows = the_setup->no_of_rows;
   disp_data->no_of_cols = the_setup->no_of_cols;
   disp_data->no_of_nodes = disp_data->no_of_rows * disp_data->no_of_cols;
   return;
   }  /*  end proc  */

short read_xmp_patterns (hop_setup_rec *the_setup,
   pattern_display_data *disp_data,path_str the_file_name,
   pattern_array the_patterns) {
   if (!read_patterns (disp_data,the_file_name,the_patterns)) return 0;
   if ((the_setup->no_of_rows != disp_data->no_of_rows) ||
      (the_setup->no_of_cols != disp_data->no_of_cols)) {
      cprintf ("\r\nnPattern rows,cols (%d,%d) from file do not agree with ",
	 disp_data->no_of_rows,disp_data->no_of_cols);
      cprintf ("\r\n   rows,cols (%d,%d) from setup.",
	  the_setup->no_of_rows,the_setup->no_of_cols);
      cprintf ("\r\n\nPress key to cancel: ");
      getch();
      return 0;
      }
   the_setup->no_of_patterns = disp_data->no_of_patterns;
   cprintf ("\r\nThere are %d exemplars, each with %d rows, and %d columns."
	    "\r\n",the_setup->no_of_patterns,the_setup->no_of_rows,
	    the_setup->no_of_cols);
   return 1;
   }  /* end function */

#define INT_ARR_END 1

int **allocate_2d_int_matrix (int rows_lower_lim,int rows_upper_lim,
   int cols_lower_lim,int cols_upper_lim) {
   /* Allocate a 2D int matrix with subscript range */
   /* rows_lower_lim..rows_upper_lim,cols_lower_lim..cols_upper_lim */
   int i,no_of_rows = rows_upper_lim - rows_lower_lim + 1,
       no_of_cols = cols_upper_lim - cols_lower_lim + 1;
   int **a;
   /* Allocate pointers to rows */
   a = (int **) malloc((size_t)((no_of_rows+INT_ARR_END)*sizeof(int*)));
   if (!a) {
      cprintf ("\r\nMemory allocation 1 failed in 2d int matrix");
      getch();
      exit (1);
      }
   a += INT_ARR_END;
   a -= rows_lower_lim;
   /* Allocate rows and set pointers to them */
   a[rows_lower_lim] = (int *) malloc
     ((size_t)((no_of_rows*no_of_cols + INT_ARR_END)*sizeof(int)));
   if (!a[rows_lower_lim]) {
      cprintf ("\r\nMemory allocation 2 failed in int 2d matrix");
      getch();
      exit(1);
      }
   a[rows_lower_lim] += INT_ARR_END;
   a[rows_lower_lim] -= cols_lower_lim;
   for (i=rows_lower_lim + 1;i <= rows_upper_lim; i++)
       a[i] = a[i-1] + no_of_cols;
   /* Return pointer to array of pointers to rows */
   return a;
   }

void free_2d_int_matrix (int **a,int rows_lower_lim,int cols_lower_lim) {
   free ((char *)(a[rows_lower_lim] + cols_lower_lim - INT_ARR_END));
   free ((char *)(a + rows_lower_lim - INT_ARR_END));
   }

void compute_hop_weights (pattern_display_data *disp_data,
   pattern_array the_patterns,int **weights) {
   int i,j,k,t;
   cprintf ("\r\nComputing weights...\r\n");
   /* Weights are allocated with indexes 1..no_of_nodes */
   for (i = 1;i<=disp_data->no_of_nodes;i++)
       for (j = 1;j <=disp_data->no_of_nodes;j++) {
	   t = 0;
	   if (i != j)
	      for (k = 0;k<disp_data->no_of_patterns;k++)
	t += (((2*the_patterns[k].nodes[i-1] - NODE_HIGH - NODE_LOW)*
		    (2*the_patterns[k].nodes[j-1] - NODE_HIGH - NODE_LOW))
			    / (NODE_HIGH - NODE_LOW));
	   weights[i][j] = t;
	   } /* i,j */
   } /* end proc */

short write_hop_weights (path_str file_name, int **w,int no_of_nodes) {
   /* Returns 0 if unsuccessful, 1 if successful  */
   FILE *the_file;
   int i,j;
   the_file = fopen (file_name,"wt"); /* Stream file, write only, text */
   if (the_file == 0) return 0;
   fprintf (the_file,"%d\n",no_of_nodes);
   for (i = 1;i <= no_of_nodes;i++)
       for (j = 1;j <= no_of_nodes;j++)
	    fprintf (the_file,"%d\n",w[i][j]);
   fclose (the_file);
   return 1;
   }

short read_hop_weights (path_str file_name,int **w,int no_of_nodes) {
   /* Note: Exemplar file must be read before weights file, to obtain */
   /*       value for no_of_nodes parameter. */

#define MAX_LINE_LEN 255
   /* Returns 0 if unsuccessful, 1 if successful  */
   FILE *the_file;
   int file_no_of_nodes;
   int i,j;
   char temp[MAX_LINE_LEN];
   the_file = fopen (file_name,"rt"); /* Stream file, read only, text */
   if (the_file == NULL) {
      cprintf ("\r\n\nProblem opening weights file %s.",file_name);
      return 0;
      }
   fgets (temp,MAX_LINE_LEN,the_file);
   if (sscanf (temp,"%d",&file_no_of_nodes) != 1) {
      cprintf ("\r\n\nProblem reading number of nodes in weights files %s.",
	  file_name);
      return 0;
      }
   if (file_no_of_nodes!= no_of_nodes) {
      cprintf ("\r\n\nWeights file number of nodes (%d) does not equal "
	 "setup value (%d).",file_no_of_nodes,no_of_nodes);
      return 0;
      }
   for (i = 1;i <= no_of_nodes;i++)
       for (j = 1;j <= no_of_nodes;j++) {
	   fgets (temp,MAX_LINE_LEN,the_file);
	   if (sscanf (temp,"%d",&w[i][j]) != 1) {
	      cprintf ("\r\n\nProblem reading weights matrix at i: %d,j: %d",
		 i,j);
	      return 0;
	      }
	   }
   fclose (the_file);
   return 1;
   }

#endif