/* File SLBRUN.C  Run Self Org Broom Balancer  */

#ifndef SLBRUN_C
#define SLBRUN_C

#include <conio.h>
#include "ut.h"
#include "slfbrm.h"

float ****w_in,***w_out;  /*  Four- and three-dimensional weight matrices */
int l_rows,l_cols,k_inp;
float frce_fact;

int xx_index[NO_OF_INP_WTS] = {X_POS_INDEX,THETA_INDEX,THETA_DOT_INDEX},
       *x_index = xx_index - 1;

   /* We use x_index to specify which components of the input vector*/
   /* are to be stored in the weight vector.  We are using components */
   /* coresponding to x position, theta, and theta dot.  Due to dimension */
   /* size limitations, we ignore x dot in the weight matrix.  */

float get_network_force (state_rec state)
   {
   float min_dist,sum = 0.0;
   int i,j,k,l,min_i,min_j,min_k;
   float xx[NO_OF_INPUTS],*x,the_force,out_force = 0.0;
   x = xx - 1;
   state_rec_to_x (state,-1.0,1.0,x);
   for (l = 1;l <= NO_OF_INP_WTS; l++)
       sum += (x[x_index[l]] - w_in[1][1][1][l])*
		   (x[x_index[l]] - w_in[1][1][1][l]);
   min_i = 1;
   min_j = 1;
   min_k = 1;
   min_dist = sum;

   /* Find minimum distance over all weights */
   for (i = 1;i <= l_rows; i++)
       for (j = 1; j <= l_cols; j++)
	   for (k = 1;k <= k_inp; k++) {
	   sum = 0;
	   for (l = 1; l <= NO_OF_INP_WTS; l++)
		    sum += (x[x_index[l]] - w_in[i][j][k][l])*
			   (x[x_index[l]] - w_in[i][j][k][l]);
	   if (sum < min_dist) {
	      min_i = i;
	      min_j = j;
	      min_k = k;
	      min_dist = sum;

	      }  /*if*/
	   } /*i,j,k*/
    the_force = frce_fact*w_out[min_i][min_j][min_k];
    out_force = the_force;
    return out_force;
    }

void run_self_org_controller(slb_setup_rec s)
   {
   tbrm_setup_rec  brm_set;
   clrscr();
   textcolor (YELLOW);
   gotoxy (1,5);
   cprintf ("Self Organizing Network Broom Balancing Controller Program\n");
   gotoxy (1,7);
   cprintf ("Press any key to start (esc to cancel): \n");
   if (getch() == ESC_KEY) return;
   initialize_brm_setup (&brm_set);
   l_rows = s.lattice_rows;
   l_cols = s.lattice_cols;
   k_inp = s.lattice_k;
   frce_fact = s.force_factor;
   init_state_rec_factors ();
   w_in = matrix4d (1,s.lattice_rows,1,s.lattice_cols,1,s.lattice_k,
	  1,NO_OF_INP_WTS);
   w_out = matrix3d (1,s.lattice_rows,1,s.lattice_cols,1,s.lattice_k);
   if (!read_weights (w_in,w_out,s)) goto exit_fn;
   clrscr();
   gotoxy (1,5);

   control_system ("Self Organizing Broom Balancer",brm_set,
      get_network_force);

exit_fn:
   free_matrix4d(w_in,1,1,1,1);
   free_matrix3d(w_out,1,1,1);
   textmode (LASTMODE);
   return;
   }

#endif