/* File SLBTRN.C Train a Self Organizing Neural Network for Broom Balancing*/

#ifndef SLBTRN_C
#define SLBTRN_C

#include <stdlib.h>
#include <math.h>  /* for fabs */
#include <conio.h>

#include "utgr.h"
#include "ut.h"
#include "slfbrm.h"

int gr_page = 0;
const float rand_x_low_lim = -1.0,
	    rand_x_up_lim = 1.0;

float gaussian (const float x,const float y,const float z) {
   return exp (-x*x - y*y - z*z);
   }

float x_factor (float x) {
   const float cutoff = 0.5;
   if (fabs(x) < cutoff) return 0.0;
   return 1.0;
   }

float theta_x_factor (float x) {
   const float cutoff = 0.5;
   if (fabs(x) < cutoff) return 1.0;
   return 0.25;
   }

float reward (slb_setup_rec s,float *x,float f,float f_factor) {
   float rxx[NO_OF_INPUTS],*rx;
   state_rec old_st,new_st;
   rx = rxx - 1;
   old_st = x_to_state_rec (x,rand_x_low_lim,rand_x_up_lim);
   force_effect (f*f_factor,old_st,&new_st);
   state_rec_to_x (new_st,rand_x_low_lim,rand_x_up_lim,rx);
   return (-fabs (rx[THETA_INDEX])
       -s.theta_dot_param*fabs(rx[THETA_DOT_INDEX])
       -s.x_pos_param*fabs(rx[X_POS_INDEX]));
   }

void write_graph_screen (float ****w, int layer,int l_rows, int l_cols,
   short color, unsigned long iters,int k1,int k2,graph_setup_rec gr_rec) {
   int i,j;
   int col,row;
   int col1,row1,col2,row2;
   int active_page,visual_page;
   gr_page++;
   visual_page = gr_page % 2;  /* % = mod operator */
   active_page = (gr_page + 1) % 2;
   setvisualpage (visual_page);
   setactivepage (active_page);
   cleardevice();
   draw_border(gr_rec);
   setcolor (color);
   for (i = 1;i <= l_rows - 1;i++) {
       for (j = 1;j <= l_cols - 1;j++) {
	   xy_to_screen_col_row (w[i][j][layer][k1],
	      w[i][j][layer][k2],&col,&row);
	   xy_to_screen_col_row (w[i][j+1][layer][k1],
	      w[i][j+1][layer][k2],&col1,&row1);
	   xy_to_screen_col_row (w[i+1][j][layer][k1],
	      w[i+1][j][layer][k2],&col2,&row2);
	   line (col,row,col1,row1);
	   line (col,row,col2,row2);
	   } /*j*/
      /*Do the last point in this row*/
      xy_to_screen_col_row (w[i][l_cols][layer][k1],
		       w[i][l_cols][layer][k2],&col,&row);
      xy_to_screen_col_row (w[i+1][l_cols][layer][k1],
		       w[i+1][l_cols][layer][k2],&col1,&row1);
      line (col,row,col1,row1);
      } /*i*/
      /*Do the last row*/
      for (j = 1;j <=l_cols - 1;j++) {
	  xy_to_screen_col_row (w[l_rows][j][layer][k1],
		     w[l_rows][j][layer][k2],&col,&row);
	  xy_to_screen_col_row (w[l_rows][j+1][layer][k1],
		     w[l_rows][j+1][layer][k2],&col1,&row1);
	  line (col,row,col1,row1);
	  } /*j*/
       graph_iter (gr_rec,iters);
       setvisualpage(active_page);
       return;
       }

void self_org_brm_train (slb_setup_rec s)
   {
   float xx[NO_OF_INPUTS],*x;
   float ****w_in,***w_out,***as;  /*  Matrices  */
   float f_new,f_incr,best_f,best_reward,the_reward;
   const no_of_fsteps = 20;
   int i,j,k,l,min_i,min_j,min_k;
   float min_dist;
   int nbhd = s.starting_nbhd;
   unsigned long iters = 0;
   float stepsize = s.starting_stepsize,min_stepsize = 0.005;
   float as_stepsize = 0.05;
   float alpha = 1.0/(NO_OF_INP_WTS*s.starting_nbhd);
   float factor;
   float sum;
   int i_lower_limit,i_upper_limit,
       j_lower_limit,j_upper_limit,
       k_lower_limit,k_upper_limit;
   int xx_index[NO_OF_INP_WTS] = {X_POS_INDEX,THETA_INDEX,THETA_DOT_INDEX},
       *x_index;

   /* We use x_index to specify which components of the input vector*/
   /* are to be stored in the weight vector.  We are using components */
   /* coresponding to x position, theta, and theta dot.  Due to dimension */
   /* size limitations, we ignore x dot in the weight matrix.  */

   x = xx - 1;  /* So x, x_index have subscript ranges 1..NO_OF_INPUTS */
   x_index = xx_index - 1;

    clrscr();
    textcolor (YELLOW);
    gotoxy (1,5);
    cprintf ("Self Organizing Network Broom Balancing Training Program\n");
    gotoxy (1,7);
    cprintf ("Press any key to start (esc to cancel): \n");
    if (getch() == ESC_KEY) return;

    w_in = matrix4d (1,s.lattice_rows,1,s.lattice_cols,1,s.lattice_k,
	 1,NO_OF_INP_WTS);
    w_out = matrix3d (1,s.lattice_rows,1,s.lattice_cols,1,s.lattice_k);

    if (s.read_wts_from_file) {
       if (!read_weights (w_in,w_out,s)) goto exit_fn;
       }
    else {
      rand_init_4d_matrix(w_in,s.lattice_rows,s.lattice_cols,s.lattice_k,
	 NO_OF_INP_WTS,1.0);
      rand_init_3d_matrix(w_out,s.lattice_rows,s.lattice_cols,s.lattice_k,1.0);
      }

    /* "as" is a matrix of search step sizes, which will be used for */
    /*  updating the output layer weight matrix.  */
    /*  Each component is initialized to 1.0.  */

    as = matrix3d (1,s.lattice_rows,1,s.lattice_cols,1,s.lattice_k);
    init_3d_matrix (as,s.lattice_rows,s.lattice_cols,s.lattice_k,1.0);

    init_state_rec_factors();

    if (!graph_setup (s.gr_rec)) return;
    draw_border(s.gr_rec);

    while (!kbhit()) {
	/* Generate uniform random input (-1 to 1) */
	for (k = 1; k <= NO_OF_INPUTS; k++)
	     x[k] = 2 * frand() - 1.0;
	/* Initialize minimum distance */
	sum = 0;
	for (k = 1;k <= NO_OF_INP_WTS; k++)
	    sum += (x[x_index[k]] - w_in[1][1][1][k])*
		   (x[x_index[k]] - w_in[1][1][1][k]);
	min_i = 1;
	min_j = 1;
	min_k = 1;
	min_dist = sum;
	/* Find minimum distance over all weights */
	for (i = 1;i <= s.lattice_rows; i++)
	    for (j = 1;j <= s.lattice_cols; j++)
		for (k = 1;k <= s.lattice_k; k++) {
		sum = 0;
		for (l = 1; l <= NO_OF_INP_WTS; l++)
		    sum += (x[x_index[l]] - w_in[i][j][k][l])*
			   (x[x_index[l]] - w_in[i][j][k][l]);
		if (sum < min_dist) {
		   min_i = i;
		   min_j = j;
		   min_k = k;
		   min_dist = sum;
		   }  /*if*/
		} /*i,j,k*/

	/* Brute force!  Search for best controlling force corresponding */
	/* to this x. */

	f_incr = 2*as[min_i][min_j][min_k]/(no_of_fsteps - 1);
	f_new = w_out[min_i][min_j][min_k] - as[min_i][min_j][min_k];
	best_f = f_new;
	best_reward = reward (s,x,best_f,s.force_factor);
	for (l=2;l<=no_of_fsteps;l++) {
	    f_new += f_incr;
	    the_reward = reward(s,x,f_new,s.force_factor);
	    if (the_reward > best_reward) {
	       best_f = f_new;
	       best_reward = the_reward;
	       }
	    }  /* l */

	/* Update weights in neighborhood around minimum */
	/* First, make sure neighborhood doesn't exceed array bounds*/

	if ((min_i - nbhd) < 1) i_lower_limit = 1;
	   else i_lower_limit = min_i - nbhd;
	if ((min_j - nbhd) < 1) j_lower_limit = 1;
	   else j_lower_limit = min_j - nbhd;
	if ((min_k - nbhd) < 1) k_lower_limit = 1;
	   else k_lower_limit = min_k - nbhd;
	if ((min_i + nbhd) > s.lattice_rows)
	   i_upper_limit = s.lattice_rows;
	   else
	   i_upper_limit = min_i + nbhd;
	if ((min_j + nbhd) > s.lattice_rows)
	   j_upper_limit = s.lattice_rows;
	   else
	   j_upper_limit = min_j + nbhd;
	if ((min_k + nbhd) > s.lattice_k)
	   k_upper_limit = s.lattice_k;
	   else
	   k_upper_limit = min_k + nbhd;
	for (i = i_lower_limit;i <= i_upper_limit; i++)
	    for (j = j_lower_limit;j <= j_upper_limit; j++)
		for (k = k_lower_limit;k <= k_upper_limit; k++) {
		    factor = gaussian (alpha*(i-min_i),alpha*(j-min_j),
			     alpha*(k-min_k));
		    factor = 1;
		    /*  Adapt input weight to selected x components */
		    for (l = 1; l <= NO_OF_INP_WTS; l++)
		    w_in [i][j][k][l] +=  factor *
			 stepsize * (x[x_index[l]] - w_in[i][j][k][l]);
		    w_out [i][j][k] +=  factor *
			 stepsize * (best_f - w_out[i][j][k]);

	}  /* i,j,k */
	as[min_i][min_j][min_k] -= as_stepsize*as[min_i][min_j][min_k];

       iters++;
       /*Display weight values on screen, when iterations = freq */

       if ((iters == 1) || ((iters %  s.freq) == 0))
	  write_graph_screen(w_in,s.graph_layer,s.lattice_cols,s.lattice_k,
	      s.color,iters,s.graph_coord_1,s.graph_coord_2,s.gr_rec);

       /*Update stepsize and nbhd size*/

       if (iters % s.iter_interval == 0) {
	  stepsize *= 0.8;
	  if (stepsize < min_stepsize) stepsize = min_stepsize;
	  nbhd -= 1;
	  if (nbhd < 1) nbhd = 1;
	  }
       }  /* while not keypressed...*/
       getch();
       outtextxy (gr_iter_col,gr_done_row,"Press key ");
       outtextxy (gr_iter_col,gr_done_row + textheight("P") + 2,"to clear.");
       if (getch() == 'p') print_4d_matrix (w_in,s.lattice_rows,
	   s.lattice_cols,s.lattice_k,NO_OF_INP_WTS);

       closegraph();
       textmode (LASTMODE);
       write_weights (w_in,w_out,s);
       free_matrix3d(as,1,1,1);
exit_fn:
       free_matrix4d(w_in,1,1,1,1);
       free_matrix3d(w_out,1,1,1);
       textmode (LASTMODE);
       gotoxy (1,1);
       return;
       }
#endif
